<?php

use VerifoneEcomAPI\ApiWrapper\Schemas\CheckoutSchema;

/**
 * Class Verifone_Checkout
 */
class Verifone_Checkout extends Verifone_Api_Entity {

	/**
	 * @var string
	 */
	private $customer_id;

	/**
	 * @var \Verifone_Helper
	 */
	private $helper;

	/**
	 * @var \VerifoneEcomAPI\ApiWrapper\Settings
	 */
	private $settings;

	/**
	 * @var \Order
	 */
	private $order;

    /**
     * @var Array
     */
	private $params;

	/**
	 * @var array
	 */
	protected $data;


	protected $precision;

	/**
	 * Verifone_Checkout constructor.
	 *
	 * @param $params
	 * @param \VerifoneEcomAPI\ApiWrapper\Settings $settings
	 * @param Verifone_Helper $helper
	 * @param string $customer_id
	 *
	 * @throws Exception
	 */
	public function __construct(
		$params,
		\VerifoneEcomAPI\ApiWrapper\Settings $settings,
		Verifone_Helper $helper,
		$customer_id = ''
	) {

        $this->params = $params;
		$this->cart       = $params['cart'];
		$this->customer_id = $customer_id;
		$this->helper      = $helper;
		$this->settings    = $settings;
		$this->precision   = $this->getCurrencyPrecision( $params['currency']->iso_code);

		$this->buildData();
	}

	/**
	 * @throws \Exception
	 */
	private function buildData() {
		$customer_data = [];
		if ( $this->customer_id !== '' ) {
			$customer_data['customer'] = $this->customer_id;
		}

		$configurations = [
			'configurations' => [
				'card' => [
					'cvv_required'       => (bool)$this->settings->getCvvOnPayment(),
					'capture_now'        => true,
					'payment_contract_id' => $this->settings->getPaymentContractId(),
				],
			],
		];

		if ( $this->settings->is3dsEnabled() ) {
			$configurations['configurations']['card']['threed_secure'] = [
				'threeds_contract_id' => $this->settings->get3dsContractId(),
				'enabled'           => $this->settings->is3dsEnabled(),
				'transaction_mode'   => $this->helper->getDeviceType(),
			];
		}
        $cart_total_paid = (float) Tools::ps_round((float) $this->cart->getOrderTotal(true, Cart::BOTH), 2);
		$amount = $this->transformToCents($cart_total_paid, $this->precision);
		$params = [
			'amount'            => $amount,
			'currency_code'      => strtoupper( $this->params['currency']->iso_code ),
			'entity_id'          => $this->settings->getEntityId(),
			// must be a string
			'merchant_reference' => sprintf( '%s', $this->cart->id ),
			'return_url'         => $this->params['returnUrl'],
            'i18n'               => $this->getLanguage(),
        ];

		$this->data = $params;
		$this->data = array_merge( $this->data, $customer_data );
		$this->data = array_merge( $this->data, $configurations );
	}

	public function transformToCents(float $inputPrice, $precision){
        $dollars = preg_replace('/[^\\d.]+/', '', $inputPrice); // get rid of currency sign
        $cents_as_float = $dollars* pow(10, $precision); //multiply by 100, it becomes float
        $cents_as_string = (string)$cents_as_float; //convert float to string
        $cents = (int) $cents_as_string; //convert string to integer
        return $cents;
    }

	/**
	 * @return array
	 */
	public function toArray() {
		return $this->data;
	}

	/**
	 * @return false|string
	 */
	public function toJson() {
		return json_encode( $this->data );
	}

	public function getCurrencyPrecision($currencyCode){
        $payment_currency_id        = Currency::getIdByIsoCode( $currencyCode );
        $payment_currency           = Currency::getCurrencyInstance( $payment_currency_id );
        $payment_currency_precision = $payment_currency->precision;
        return $payment_currency_precision;
    }

    /**
     * @return array
     */
    private function getLanguage() {
        $lang = Language::getLanguage((int) Configuration::get( 'PS_LANG_DEFAULT' ))['iso_code'];
        return [
            'default_language' => in_array( $lang, CheckoutSchema::SUPPORTED_I18N_LANGUAGES ) ? $lang : 'en',
            'fallback_language' => 'en',
            'show_language_options' => true,
        ];
    }

}
